"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _createStarExport(obj) { Object.keys(obj) .filter((key) => key !== "default" && key !== "__esModule") .forEach((key) => { if (exports.hasOwnProperty(key)) { return; } Object.defineProperty(exports, key, {enumerable: true, configurable: true, get: () => obj[key]}); }); }// src/enhance.ts



var _axios = require('axios'); var _axios2 = _interopRequireDefault(_axios); _createStarExport(_axios);

// src/slots/cache-slot.ts
var _clone = require('clone'); var _clone2 = _interopRequireDefault(_clone);

// src/libs/clone-response.ts

var cloneResponse = (response, config) => {
  const next = Object.assign({}, response);
  next.data = _clone2.default.call(void 0, next.data, false);
  next.headers = _clone2.default.call(void 0, next.headers, false);
  next.config = config;
  return next;
};

// src/libs/merge-slot-options.ts
var mergeSlotOptions = (globalOptions, localOptions) => {
  const globalOpts = typeof globalOptions === "undefined" ? {} : typeof globalOptions === "boolean" ? { enable: globalOptions } : { enable: true, ...globalOptions };
  const localOpts = typeof localOptions === "undefined" ? {} : typeof localOptions === "boolean" ? { enable: localOptions } : { enable: true, ...localOptions };
  return Object.assign({ enable: true }, globalOpts, localOpts);
};

// src/slots/cache-slot.ts
var _CacheSlot = class _CacheSlot {
  constructor(options = false) {
    this.options = options;
    this.cacheMap = {};
  }
  hit(config, newCache) {
    const options = mergeSlotOptions(this.options, config.cache);
    const { allowedMethods = _CacheSlot.defaultAllowedMethods, validate } = options;
    const enable = options.enable !== false && allowedMethods.includes(config.method.toLowerCase()) && (!validate || validate(config));
    if (!enable) {
      return newCache(config);
    }
    const { maxAge = _CacheSlot.defaultMaxAge, format } = options;
    const formatConfig = _CacheSlot.getFormatConfig(config);
    const key = JSON.stringify(
      format ? format(_clone2.default.call(void 0, formatConfig, false)) : formatConfig
    );
    const cacheData = this.cacheMap[key];
    if (cacheData) {
      if (cacheData.time + maxAge >= Date.now()) {
        return Promise.resolve(cloneResponse(cacheData.response, config));
      }
      delete this.cacheMap[key];
    }
    return newCache(config).then((response) => {
      this.cacheMap[key] = {
        time: Date.now(),
        response: cloneResponse(response, response.config)
      };
      return response;
    });
  }
  clear(filter) {
    if (!filter) {
      this.cacheMap = {};
      return;
    }
    const keys = Object.keys(this.cacheMap);
    for (let i = keys.length, key; i-- > 0; ) {
      key = keys[i];
      if (filter(JSON.parse(key))) {
        delete this.cacheMap[key];
      }
    }
  }
  static getFormatConfig(config) {
    return this.formatKeys.reduce(
      (carry, key) => {
        carry[key] = config[key];
        return carry;
      },
      {}
    );
  }
};
_CacheSlot.defaultMaxAge = 10 * 60 * 1e3;
_CacheSlot.formatKeys = ["baseURL", "url", "method", "params", "data", "headers"];
_CacheSlot.defaultAllowedMethods = ["get"];
var CacheSlot = _CacheSlot;

// src/libs/prevent-transform.ts
var preventTransform = (promise, transform) => {
  return {
    then() {
      transform.push.apply(transform, arguments);
      return promise;
    }
  };
};

// src/libs/override-request.ts

var overrideRequest = (instance) => {
  Object.keys(_axios.Axios.prototype).forEach((key) => {
    if (typeof _axios.Axios.prototype[key] === "function") {
      Object.defineProperty(instance, key, {
        get() {
          return _axios.Axios.prototype[key].bind(instance);
        },
        enumerable: true
      });
    }
  });
  instance.request = function overrideRequest2(cfg) {
    let shouldUnwrap = true;
    const config = cfg || {};
    const promise = _axios.Axios.prototype.request.call(instance, config).then((response) => {
      return shouldUnwrap ? response.data : response;
    });
    promise.toRaw = function toRawResponse() {
      shouldUnwrap = false;
      return promise;
    };
    return promise;
  };
};

// src/slots/retry-slot.ts

var _RetrySlot = class _RetrySlot {
  constructor(options) {
    this.options = options;
    this.resolvingAuthorized = {
      refreshTime: 0,
      handler: null
    };
  }
  async validate(err, config, currentAttempt) {
    const {
      enable,
      gap = _RetrySlot.GAP,
      maxAttempt = _RetrySlot.MAX_ATTEMPT,
      allowedMethods = _RetrySlot.ALLOWED_METHODS,
      allowedHttpStatus = _RetrySlot.ALLOWED_HTTP_STATUS,
      resolveUnauthorized,
      onAuthorized,
      validate
    } = mergeSlotOptions(this.options, config.retry);
    if (enable === false) return false;
    if (currentAttempt > maxAttempt) return false;
    if (_axios2.default.isCancel(err)) return false;
    if (
      // 当前请求主动设置了重试配置，则必须忽略method判断
      config.retry === void 0 && !allowedMethods.includes(config.method.toLowerCase())
    ) {
      return false;
    }
    const waitingAuthorize = err.response && err.response.status === 401 && typeof resolveUnauthorized === "function" && typeof onAuthorized === "function";
    if (err.response && !waitingAuthorize && !this.isAllowedStatus(err.response, allowedHttpStatus)) {
      return false;
    }
    if (validate && !validate(config)) return false;
    if (waitingAuthorize) {
      if (this.resolvingAuthorized.handler) {
        try {
          const resolved = await this.resolvingAuthorized.handler;
          if (!resolved) return false;
          await onAuthorized(config);
        } catch (e2) {
          return false;
        }
      } else if (this.resolvingAuthorized.refreshTime > config.timestamp) {
        await onAuthorized(config);
      } else {
        try {
          this.resolvingAuthorized.handler = resolveUnauthorized(err);
          const resolved = await this.resolvingAuthorized.handler;
          if (!resolved) return false;
          this.resolvingAuthorized.refreshTime = Date.now();
          await onAuthorized(config);
        } catch (e) {
          console.error(e);
          return false;
        } finally {
          this.resolvingAuthorized.handler = null;
        }
      }
    }
    const durationMS = err.response ? this.getRetryAfter(err.response.headers, gap) : gap;
    return new Promise((resolve) => {
      setTimeout(resolve, durationMS, true);
    });
  }
  getRetryAfter(headers, min) {
    const retryAfter = headers["Retry-After"] || headers["retry-after"];
    if (!retryAfter) return 0;
    if (Number.isFinite(Number(retryAfter))) {
      return Math.max(min, Number(retryAfter) * 1e3);
    }
    const timestamp = new Date(retryAfter).getTime();
    if (Number.isInteger(timestamp)) {
      return Math.max(min, timestamp - Date.now());
    }
    return 0;
  }
  isAllowedStatus(response, allowedHttpStatus) {
    const currentStatus = response.status;
    return allowedHttpStatus.some((range) => {
      return typeof range === "number" ? range === currentStatus : currentStatus >= range[0] && currentStatus <= range[1];
    });
  }
};
_RetrySlot.ALLOWED_METHODS = [
  "get",
  "head",
  "put",
  "patch",
  "delete"
];
_RetrySlot.ALLOWED_HTTP_STATUS = [
  [100, 199],
  429,
  [500, 599]
];
_RetrySlot.MAX_ATTEMPT = 3;
_RetrySlot.GAP = 500;
var RetrySlot = _RetrySlot;

// src/slots/throttle-slot.ts




var _ThrottleSlot = class _ThrottleSlot {
  constructor(options) {
    this.options = options;
    this.threads = {};
  }
  hit(config, newThread) {
    const options = mergeSlotOptions(this.options, config.throttle);
    const {
      allowedMethods = _ThrottleSlot.defaultAllowedMethods,
      format,
      validate
    } = options;
    const enable = options.enable !== false && allowedMethods.includes(config.method.toLowerCase()) && (!validate || validate(config));
    if (!enable) {
      return newThread(config);
    }
    const formatConfig = _ThrottleSlot.getFormatConfig(config);
    const key = JSON.stringify(
      format ? format(_clone2.default.call(void 0, formatConfig, false)) : formatConfig
    );
    const thread = this.threads[key];
    if (thread) {
      return thread.then((response) => cloneResponse(response, config)).catch((err) => {
        return Promise.reject(
          _axios2.default.isCancel(err) ? err : new (0, _axios.AxiosError)(
            err.message,
            err.code,
            config,
            err.request,
            err.response ? cloneResponse(err.response, config) : void 0
          )
        );
      });
    }
    const promise = this.threads[key] = newThread(config);
    const clearThread = () => {
      delete this.threads[key];
    };
    promise.then(clearThread, clearThread);
    return promise;
  }
  static getFormatConfig(config) {
    return this.formatKeys.reduce(
      (carry, key) => {
        carry[key] = config[key];
        return carry;
      },
      {}
    );
  }
};
_ThrottleSlot.formatKeys = [
  "baseURL",
  "url",
  "method",
  "params",
  "data",
  "headers",
  "timeout",
  "maxContentLength",
  "maxBodyLength",
  "xsrfCookieName",
  "xsrfHeaderName"
];
_ThrottleSlot.defaultAllowedMethods = [
  "get",
  "head",
  "put",
  "patch",
  "delete"
];
var ThrottleSlot = _ThrottleSlot;

// src/slots/request-slot.ts



var RequestSlot = class {
  constructor(defaultAdapter, getHttpStatus) {
    this.defaultAdapter = defaultAdapter;
    this.getHttpStatus = getHttpStatus;
    if (!defaultAdapter) {
      throw new Error("axios default adapter is not found.");
    }
  }
  hit(config, [onResolve, onReject], shouldRetry) {
    const adapter = this.defaultAdapter;
    const loop = (currentAttempt) => {
      return adapter(config).then(onResolve, onReject).then((response) => {
        const getHttpStatus = config.getHttpStatus || this.getHttpStatus;
        if (config.validateStatus && getHttpStatus) {
          const realHttpStatus = getHttpStatus(response);
          response.status = realHttpStatus;
          if (!config.validateStatus(realHttpStatus)) {
            return Promise.reject(
              new (0, _axios.AxiosError)(
                "Request failed with status code " + realHttpStatus,
                void 0,
                response.config,
                response.request,
                response
              )
            );
          }
        }
        return response;
      }).catch((err) => {
        return shouldRetry(err, config, currentAttempt + 1).then(
          (enable) => {
            return enable ? loop(currentAttempt + 1) : Promise.reject(err);
          },
          () => Promise.reject(err)
        );
      });
    };
    return loop(0);
  }
};

// src/enhance.ts
var enhance = (instance, options = {}) => {
  overrideRequest(instance);
  let defaultAdapter = instance.defaults.adapter;
  if (typeof defaultAdapter === "function") {
    defaultAdapter = defaultAdapter.__prev_adapter__ || defaultAdapter;
  }
  const cache = new CacheSlot(options.cache);
  const throttle = new ThrottleSlot(options.throttle);
  const request = new RequestSlot(_axios.getAdapter.call(void 0, defaultAdapter), options.getHttpStatus);
  const retry = new RetrySlot(options.retry);
  const validateRetry = retry.validate.bind(retry);
  const focaAdapter = function focaAdapter2(config) {
    config.timestamp = Date.now();
    const transformHandler = [];
    const promise = Promise.resolve().then(() => {
      return cache.hit(config, () => {
        return throttle.hit(config, () => {
          return request.hit(config, transformHandler, validateRetry);
        });
      });
    });
    return preventTransform(promise, transformHandler);
  };
  focaAdapter.__prev_adapter__ = defaultAdapter;
  instance.defaults.adapter = focaAdapter;
  instance.clearCache = cache.clear.bind(cache);
  return instance;
};
var axios3 = enhance(_axios2.default, _axios2.default.defaults);
axios3._ = axios3.create;
axios3.create = (config = {}) => {
  const { retry, throttle, cache, getHttpStatus, ...rest } = config;
  return enhance(axios3._(rest), {
    retry,
    throttle,
    cache,
    getHttpStatus
  });
};

// src/index.ts





exports.axios = axios3; exports.default = axios3; exports.enhance = enhance;
//# sourceMappingURL=index.js.map